.\" file MASS/polr.d
.\" copyright (C) 1994-9 W. N. Venables and B. D. Ripley
.\"
.BG
.FN polr
.TL
Proportional Odds Logistic Regression
.DN
Fits a proportional odd logistic regression model to an ordered factor
response.
.CS
polr(formula, data=NULL, weights, start, ..., subset, 
     na.action=na.fail, contrasts=NULL, Hess=F)
.RA
.AG formula
a formula expression as for regression models, of the form 
`response ~ predictors'. The response should be a factor (preferably an ordered
factor), which will be interpreted as an ordinal response, with levels
ordered as in the factor.  A proportional odds model will be
fitted. The model must have an intercept: attempts to remove one will
lead to a warning and be ignored. An offset may be used. See the
documentation of `formula' for other details.
.OA
.AG data
an optional data frame in which to interpret the variables occurring
in `formula'.
.AG weights
optional case weights in fitting.  Default to 1.
.AG start
initial values for the parameters.
.AG ...
additional arguments (currently ignored).
.AG subset
expression saying which subset of the rows of the data should  be used
in the fit. All observations are included by default.
.AG na.action
a function to filter missing data.
.AG contrasts
a list of contrasts to be used for some or all of
the factors appearing as variables in the model formula.
.AG Hess
logical for whether the Hessian (the observed information matrix)
should be returned.
.RT
A object of class `"polr"'.
.RC coefficients
the coefficients of the linear predictor.
.RC zeta
the intercepts for the class boundaries.
.RC deviance
the residual deviance.
.RC fitted.values
a matrix, with a column for each level of the response.
.RC lev
the names of the response levels.
.RC terms
the `terms' structure describing the model.
.RC df.residual
the number of residual degrees of freedoms, calculated using the weights.
.RC edf
the (effective) number of degrees of freedom used by the model
.RC n
the (effective) number of observations,  calculated using the weights
.RC call
the matched call.
.RC convergence
the convergence message returned by `nlminb'.
.RC niter
the number of iterations, function and gradient evaluations used by `nlminb'.
.RC Hessian
(if `Hess' is true).
.SA
`nlminb', `glm', `multinom'.
.EX
options(contrasts=c("contr.treatment", "contr.poly"))
house.plr <- polr(Sat ~ Infl + Type + Cont, weights = Freq, data = housing)
house.plr
summary(house.plr)
predict(house.plr,housing, type="p")
addterm(house.plr, ~.^2, test = "Chisq")
house.plr2 <- stepAIC(house.plr, ~.^2)
house.plr2$anova
.KW models
.WR
